/* TIMER_INT routine(s) */

#if defined(__AVR__)
    #include <avr/io.h>
    #include <avr/pgmspace.h>
    #include <avr/delay.h>

    #include "i2c_master.h"

#endif
#if defined(__ets__)
    #include <SmingCore/Digital.h>
    #include <Libraries/OneWire/OneWire.h>
    #include <SmingCore/FileSystem.h>
    #include "crc8.h"
    #include "onewire.h"
    #include "lcd.h"
    #include "DS18X20.h"
    #include "virtual_lcd.h"
    
    #include "do_1s_stuff.h"
#if defined(NODEMCU_HARDWARE_TIMER)
    #include <SmingCore/HardwareTimer.h>
#endif
#endif

#include "defines.h"
#include "ones_count.h"

#include "keyboard_filter.h"

#include "timer_int.h"

/*--------------------------------------------------------*/

#if defined(__AVR__)

void get_sts_data_AVR(void) {

	    STS_Ready = i2c_start(0x95); //(I2C_STS_ADDRESS)); // << 1) | 0x01);
	    STS_Ready = ~STS_Ready & 0x01;
//
	    if (STS_Ready != 0) {
			    STS_data[0] = i2c_read_ack();
			    STS_data[1] = i2c_read_ack();
			    STS_data[2] = i2c_read_nack();
			    }
		else	    { /* STS not ready */ } 
	    i2c_stop(); // STOP

//	    if (sensirion_crc8(&STS_byte1, 3) == 0) {
		i2c_start(0x94); //I2C_STS_ADDRESS << 1);
		i2c_write(0xF3); // No Hold Master STS Start T
//						    }
//	    else	{
//		i2c_start(0x94);
//		i2c_write(0xFE); // soft RESET
//			}
	    i2c_stop();

			}
#endif

/*--------------------------------------------------------*/
/* every 10ms - TIMER 1 int    							  */
/* exact on AVRs, 20% error on ESP8266 - for software INT */
/*--------------------------------------------------------*/

#if defined(__AVR__)
    ISR (TIMER1_COMPA_vect)
#elif defined(__ets__)
    #if defined(NODEMCU_HARDWARE_TIMER)
	void IRAM_ATTR 	Timer_1_ISR(void) 
    #else
	void 		Timer_1_ISR(void) 
    #endif
#else
    #error("Architecture NOT supported")
#endif

{

/* keyboard filtering - platform independent */
/* ------------------------------------------*/
#if defined(__AVR__) // for ESP routines in Timer_2_ISR

    keyboard_filter();
    
#endif // defined(__AVR__)

/*--------------------------------------------------------------------*/
/* u_time counts 10ms intervals, max value = 99 */
	u_time++;
	if ((u_time / 10) < 8) { if (alarm_type != 0) {
				 if (alarm_type == 0) {  buzzer_mask = 0x00; }
				 if ((alarm_type & (COLUMN_ALARM_TYPE | HEAD_ALARM_TYPE)) != 0) { buzzer_mask |= 0x05; }
				 if ((alarm_type & (COLUMN_ALARM_TYPE | HEAD_ALARM_TYPE)) == 0)   { buzzer_mask &= ~0x05; 				   }
				 if ((alarm_type & (PREHEAT_COLUMN_ALARM_TYPE | WATER_ALARM_TYPE)) != 0) { buzzer_mask |= 0xF0; }
				 if ((alarm_type & (PREHEAT_COLUMN_ALARM_TYPE | WATER_ALARM_TYPE)) == 0) {  buzzer_mask &= ~0xF0; }
				 if ((alarm_type & TIMER_ALARM_TYPE) != 0) {  buzzer_mask |= 0x75; }

				 buzzer_mask >>= (u_time / 10);
				    #if !defined(HEARTBEAT_LED)
				#if (defined(PRODUCTION_BOARD) || defined(PRODUCTION_BOARD_BY_KRZYSKI) || defined(PRODUCTION_BOARD_BY_NEMO))
				    if ((buzzer_mask & 0x01) != 0)  {
					        LED_1_PORT &= ~_BV(LED_1_BIT);
									    }
				    else { LED_1_PORT |= _BV(LED_1_BIT); }
				#endif
				#if defined(DEV_BOARD_NANO)
				    if ((buzzer_mask & 0x01) != 0)  {
					LED_1_PORT |= _BV(LED_1_BIT);
								}
				else { LED_1_PORT &= ~_BV(LED_1_BIT); }
				#endif
				#if (defined(DEV_BOARD_NODE_MCU) || defined(PRODUCTION_BOARD_WEMOS))
				if ((buzzer_mask & 0x01) != 0)  {
					digitalWrite(LED_1_PIN, 0); }
				else { digitalWrite(LED_1_PIN, 1);
								    }
				#endif
				#endif
						} // if alarm_type != 0
			    } // if u_time?
	else 		
		#if !defined(HEARTBEAT_LED)
		    #if defined(PRODUCTION_BOARD) || defined(PRODUCTION_BOARD_BY_KRZYSKI) || defined(PRODUCTION_BOARD_BY_NEMO)
			{ LED_1_PORT |= _BV(LED_1_BIT); }
		    #endif
		    #if defined(DEV_BOARD_NANO)
			{ LED_1_PORT &= ~_BV(LED_1_BIT); }
		    #endif
		    #if (defined(DEV_BOARD_NODE_MCU) || defined(PRODUCTION_BOARD_WEMOS))
			{ digitalWrite(LED_1_PIN, 1); }
		    #endif
		#endif

/* every 100ms */
    if (u_time % 10 == 0) { flagi |= MS100_FLAG; 
			} /* /every 100ms */

    if (u_time == 50)  { flagi |= MS500_FLAG; }
    /* 1s elapsed */
    if (u_time == 100) { u_time = 0;
			flagi |= S1_FLAG | MS500_FLAG;

		    #if defined(HEARTBEAT_LED)
			#if defined(__AVR__)
			    LED_1_PORT ^= _BV(LED_1_BIT);
			#endif
			#if defined(__ets__)
			    digitalWrite(LED_1_PIN, state);
			    state = !state;
			#endif
		    #endif

/* ------------------------------------------------------------------------ */

#if defined(__AVR__)
/* STS-21 */
    #if defined(I2C_IO)
//	get_sts_data_AVR();
    #endif
#endif
/*  /STS-21 */

/* ------------------------------------------------------------------------ */

    /* RTC */
#if defined INCLUDE_RTC
		       second++;
		       if (second > 59) { second = 0;
					  minute++;
					  if (minute > 59) { minute = 0;
							     hour++;
							     if (hour > 23) { hour = 0; }
							   }
					}
#endif

/* ------------------------------------------------------------------------ */

#if defined DOWNCOUNT_TIMER
			if ((flagi3 & DOWNCOUNT_TIMER_STARTED_FLAG) != 0) {
			    downcount_timer_second--;
			    if (downcount_timer_second == 0xFF) { 
					  downcount_timer_second = 59;
					  downcount_timer_minute--;
					  if (downcount_timer_minute == 0xFF) { 
						downcount_timer_minute = 59;
						downcount_timer_hour--;
									      }
								}
			    if ((downcount_timer_second == 0) && 
				(downcount_timer_minute == 0) && 
				(downcount_timer_hour == 0)) {
				flagi3 &= ~DOWNCOUNT_TIMER_STARTED_FLAG;
				flagi3 &= ~PWM_ENABLED_FLAG;

				if ((AUTO_PWM_STARTED_FLAG_VARIABLE & AUTO_PWM_STARTED_FLAG) == 0) {
				    alarm_type |= TIMER_ALARM_TYPE;				   }
				else {
				    alarm_type |= COLUMN_ALARM_TYPE;
				    AUTO_PWM_STARTED_FLAG_VARIABLE &= ~AUTO_PWM_STARTED_FLAG;
				     }

				// restore downcounter start value
			#if defined(__AVR__)
				downcount_timer_hour = pgm_read_byte_near((PGM_P)&time_table[encoder_position][0]);
				downcount_timer_minute = pgm_read_byte_near((PGM_P)&time_table[encoder_position][1]);
				downcount_timer_second = pgm_read_byte_near((PGM_P)&time_table[encoder_position][2]);
			#endif
			#if defined(__ets__)
				downcount_timer_hour = time_table[encoder_position][0];
				downcount_timer_minute = time_table[encoder_position][1];
				downcount_timer_second = time_table[encoder_position][2];
			#endif
							     }
									  }
#endif

/* ------------------------------------------------------------------------ */

#if defined AUTO_PWM_DOWNCOUNT_TIMER
			if ((flagi3 & AUTO_PWM_DOWNCOUNT_TIMER_STARTED_FLAG) != 0) {
		switch (virt_lcd_line2[32]) {
		    case 'S':  	{ virt_lcd_line2[32] = 's';
				  break;
				}
		    case 's': 	{ virt_lcd_line2[32] = 'S';
				  break;
				}
		    case 'U': 	{ virt_lcd_line2[32] = 'u'; 
				  break;
				}
		    case 'u':	{ virt_lcd_line2[32] = 'U'; 
				  break;
				}
					    }
			    auto_pwm_downcount_timer_second--;
			    if (auto_pwm_downcount_timer_second == 0xFF) { 
					  auto_pwm_downcount_timer_second = 59;
					  auto_pwm_downcount_timer_minute--;
									 }
			    if ((auto_pwm_downcount_timer_second == 0) && 
				(auto_pwm_downcount_timer_minute == 0)) {
// ACTION to be taken AFTER AUTO_PWM downcount timer expires
				flagi3 &= ~AUTO_PWM_DOWNCOUNT_TIMER_STARTED_FLAG;
				if ((flagi3 & COLUMN_STABILIZED_FLAG) != 0) {
				    switch (auto_pwm_channel) {
					case 0:	flagi3 |= PWM_ENABLED_FLAG;
						break;
					case 1:	flagi4 |= PWM2_ENABLED_FLAG;
						break;
							      }
									    }
				// restore downcounter start value
			#if defined(__AVR__)
				auto_pwm_downcount_timer_minute = AUTO_PWM_RELOAD_MINUTE;
				auto_pwm_downcount_timer_second = AUTO_PWM_RELOAD_SECOND;
			#endif
			#if defined(__ets__)
				auto_pwm_downcount_timer_minute = AUTO_PWM_RELOAD_MINUTE; //time_table[encoder_position][1];
				auto_pwm_downcount_timer_second = AUTO_PWM_RELOAD_SECOND; //time_table[encoder_position][2];
			#endif
									}
										  }
#endif

/* ------------------------------------------------------------------------ */

#if defined(PWM_TIMER)
			pwm_timer++;
			if (pwm_timer > 99) { pwm_timer = 0; }
	#if !defined(I2C_IO)
	    #if defined(__AVR__)
			if ((pwm_output == 0) || ((flagi3 & PWM_ENABLED_FLAG) == 0)) { KEY_5_PORT &= ~_BV(KEY_5_BIT); }
	    #else
			if ((pwm_output == 0) || ((flagi3 & PWM_ENABLED_FLAG) == 0)) { digitalWrite(KEY_5_PIN,0); }
	    #endif
	#else
	    #if defined(PWM_BIT_INVERTED)
			if ((pwm_output == 0) || ((flagi3 & PWM_ENABLED_FLAG) == 0)) { i2c_output |= _BV(PWM_BIT); }
	    #else
			if ((pwm_output == 0) || ((flagi3 & PWM_ENABLED_FLAG) == 0)) { i2c_output &= ~_BV(PWM_BIT); }
	    #endif
	#endif	// I2C_IO

			if ((flagi3 & PWM_ENABLED_FLAG) != 0) {
	#if !defined(I2C_IO)
			#if defined(__AVR__)
				if (pwm_output == 100) { KEY_5_PORT |= _BV(KEY_5_BIT); }
			#else
				if (pwm_output == 100) { digitalWrite(KEY_5_PIN,1); }
			#endif
				if ((pwm_output != 0) && (pwm_output != 100)) {
				    if (pwm_output >= 10) {
			#if defined(__AVR__)
				    if ((pwm_timer % 10) < (pwm_output / 10)) { KEY_5_PORT |= _BV(KEY_5_BIT); }
			#else // ESP
				    if ((pwm_timer % 10) < (pwm_output / 10)) { digitalWrite(KEY_5_PIN,1); }
			#endif
					    else 
			#if defined(__AVR__)
					{ KEY_5_PORT &= ~_BV(KEY_5_BIT); }
			#else // ESP
					{ digitalWrite(KEY_5_PIN,0); }
			#endif
							  }
				    else    {
					if (pwm_timer  < pwm_output) 
				    #if defined(__AVR__) 
					    { KEY_5_PORT |= _BV(KEY_5_BIT); }
				    #else
					    {digitalWrite(KEY_5_PIN, 1); }
				    #endif
					    else
				    #if defined(__AVR__)
					    { KEY_5_PORT &= ~_BV(KEY_5_BIT); }
				    #else
					    { digitalWrite(KEY_5_PIN, 0); }
				    #endif
					    }
									      }
	#else // I2C_IO
		#if !defined(__ets__) // ESP

			#if defined(PWM_BIT_INVERTED)
				if (pwm_output == 100) { i2c_output &= ~_BV(PWM_BIT); }
				    else { if (pwm_output != 0) {
					if (pwm_output >= 10) {
					    if ((pwm_timer % 10) < (pwm_output / 10)) { i2c_output &= ~_BV(PWM_BIT); }
						else 				  { i2c_output |= _BV(PWM_BIT); }
							      }
					else    {
					    if (pwm_timer  < pwm_output) { i2c_output &= ~_BV(PWM_BIT); }
						else 			 { i2c_output |= _BV(PWM_BIT); }
						}
								}
					 }
			#else
				if (pwm_output == 100) { i2c_output |= _BV(PWM_BIT); }
				    else { if (pwm_output != 0) {
					if (pwm_output >= 10) {
					    if ((pwm_timer % 10) < (pwm_output / 10)) { i2c_output |= _BV(PWM_BIT); }
						else 				  { i2c_output &= ~_BV(PWM_BIT); }
							      }
					else    {
					    if (pwm_timer  < pwm_output) { i2c_output |= _BV(PWM_BIT); }
						else 			 { i2c_output &= ~_BV(PWM_BIT); }
						}
								}
					 }
			#endif
		#else
			#if defined(PWM_BIT_INVERTED)
				if (pwm_output == 100) { i2c_output3 &= ~_BV(PWM_BIT); }
				    else { if (pwm_output != 0) {
					if (pwm_output >= 10) {
					    if ((pwm_timer % 10) < (pwm_output / 10)) { i2c_output3 &= ~_BV(PWM_BIT); }
						else 				  { i2c_output3 |= _BV(PWM_BIT); }
							      }
					else    {
					    if (pwm_timer  < pwm_output) { i2c_output3 &= ~_BV(PWM_BIT); }
						else 			 { i2c_output3 |= _BV(PWM_BIT); }
						}
								}
					 }
			#else
				if (pwm_output == 100) { i2c_output3 |= _BV(PWM_BIT); }
				    else { if (pwm_output != 0) {
					if (pwm_output >= 10) {
					    if ((pwm_timer % 10) < (pwm_output / 10)) { i2c_output3 |= _BV(PWM_BIT); }
						else 				  { i2c_output3 &= ~_BV(PWM_BIT); }
							      }
					else    {
					    if (pwm_timer  < pwm_output) { i2c_output3 |= _BV(PWM_BIT); }
						else 			 { i2c_output3 &= ~_BV(PWM_BIT); }
						}
								}
					 }
			#endif

		#endif
	#endif
	 /* I2C_IO */
								}
#endif

/* ------------------------------------------------------------------------ */

#if defined(PWM2_TIMER)

// On AVR i2o_output on ESP i2c_output3

#if defined(I2C_IO)
			pwm2_timer++;
			if (pwm2_timer > 99) { pwm2_timer = 0; }
		#if !defined(__ets__)
			if ((pwm2_output == 0) || ((flagi4 & PWM2_ENABLED_FLAG) == 0)) { i2c_output &= ~_BV(PWM2_BIT); }
		#else
			if ((pwm2_output == 0) || ((flagi4 & PWM2_ENABLED_FLAG) == 0)) { i2c_output3 &= ~_BV(PWM2_BIT); }
		#endif
			if ((flagi4 & PWM2_ENABLED_FLAG) != 0) {
		#if !defined(__ets__)
				if (pwm2_output == 100) { i2c_output |= _BV(PWM2_BIT); }
		#else
				if (pwm2_output == 100) { i2c_output3 |= _BV(PWM2_BIT); }
		#endif
				    else { if (pwm2_output != 0) {
					if (pwm2_output >= 10) {
		#if !defined(__ets__)
					    if ((pwm2_timer % 10) < (pwm2_output / 10)) { i2c_output |= _BV(PWM2_BIT); }
						else 				  { i2c_output &= ~_BV(PWM2_BIT); }
		#else
					    if ((pwm2_timer % 10) < (pwm2_output / 10)) { i2c_output3 |= _BV(PWM2_BIT); }
						else 				  { i2c_output3 &= ~_BV(PWM2_BIT); }
		#endif
							      }
					else    {
		#if !defined(__ets__)
					    if (pwm2_timer  < pwm2_output) { i2c_output |= _BV(PWM2_BIT); }
						else 			 { i2c_output &= ~_BV(PWM2_BIT); }
		#else
					    if (pwm2_timer  < pwm2_output) { i2c_output3 |= _BV(PWM2_BIT); }
						else 			 { i2c_output3 &= ~_BV(PWM2_BIT); }
		#endif
						}
								}
					 }
								}
#endif
#endif

		     } /* 1s elapsed IF */
} // 10ms ISR

/* ------------------------------------------------------------ */
/* keyboard filtering, within SOFT interrupt, ESP8266 platform  */

#if defined(__ets__) // for ESP routines in Timer_2_ISR

    void Timer_2_ISR(void)  { 
	keyboard_filter();
// DEBUG
/*	#if defined(I2C_IO)
			if ((flagi3 & PWM_ENABLED_FLAG) != 0) {
				if (pwm_output == 100) { i2c_output &= ~_BV(LCD_BACKLIGHT); }
				    else { if (pwm_output != 0) {
					if (pwm_output >= 10) {
					    if ((pwm_timer % 10) < (pwm_output / 10)) { i2c_output &= ~_BV(LCD_BACKLIGHT); }
						else 				  { i2c_output |= _BV(LCD_BACKLIGHT); }
							      }
					else    {
					    if (pwm_timer  < pwm_output) { i2c_output &= ~_BV(LCD_BACKLIGHT); }
						else 			 { i2c_output |= _BV(LCD_BACKLIGHT); }
						}
								}
					 }
							      }
	#endif
*/
	 /* I2C_IO */
			    }

#endif // defined(__ets__)

/* ------------------------------------------------------------ */

#if defined(__ets__)
void Timer_500ms_ISR(void) {
if ((flagi & IN_CONFIG_FLAG) != 0) {
// display "config welcome" screen
    if ((second < 3) && (hour == 0) && (minute == 0)) {
	lcd.setCursor(0,0);
	lcd.print("Tryb konfigur.");
	lcd.setCursor(0,1);
	lcd.print("Liczba czujn.:");

	num_of_sensors = 0;

	lcd.send((num_of_sensors + '0'), DATA);
						      }
// main config loop
    if ((second >= 3) && (hour >= 0) && (minute >= 0)) {
	if (wait_key_release == 0) {
	    lcd.setCursor(0,0);
	    switch (num_of_sensors) {
			    case 0:
				    lcd.send('T', DATA);
				    for (i = 0; i < 7; i++) {
					lcd.send(temperature_labels[0][i], DATA);
							    }
				    break;

				case 1:
				    lcd.send('T', DATA);
				    for (i = 0; i < 7; i++) {
					lcd.send(temperature_labels[1][i], DATA);
							    }
				    break;

			    case 2:
				    lcd.send(' ', DATA);
				    for (i = 0; i < 7; i++) {
					lcd.send(temperature_labels[2][i], DATA);
							    }
				    break;

			    case 3:
				    lcd.send(' ', DATA);
				    for (i = 0; i < 7; i++) {
					lcd.send(temperature_labels[3][i], DATA);
							    }
				    break;
				} // switch
	    lcd.print(":        ");

	    OneWire_bus.reset();
	    OneWire_bus.write(OW_READ_ROM);
	    OneWire_bus.read_bytes(&scratchpad_table[num_of_sensors][0], 8);
	    j = crc8(&scratchpad_table[num_of_sensors][0],8);

	    lcd.setCursor(9,0);
	    if ( j == 0 ) { lcd.print("OK");
		for ( i = 0; i < 8; i++) {
		    sensor_addreses[num_of_sensors][i] = scratchpad_table[num_of_sensors][i];
					 }
			  }
		    else	  { lcd.print("--"); }
	    lcd.setCursor(0,1);
	    lcd.print("Numer  czujn.:");
	    lcd.send((num_of_sensors + '1'), DATA);

	    #if defined(KEY_1_PIN)
		if ((soft_keys & SOFT_KEY_1_MASK) != 0 ) {
	    #else
//		if ((soft_keys & SOFT_KEY_2_MASK) != 0 ) {
//		using GPIO16 / KEY_2 causes hang.
		if ((soft_keys3 & SOFT_ENCODER_SWITCH_MASK) != 0 ) {
	    #endif
	// write DS address and advance
    file_t	config_file;
	    config_file = fileOpen("sensors_addresses.bin", eFO_ReadWrite);
	    fileSeek(config_file, num_of_sensors * 8, eSO_FileStart);
	    fileWrite(config_file, &sensor_addreses[num_of_sensors][0], 8);
	    fileClose(config_file);
	    config_file = fileOpen("num_of_sensors.bin", eFO_ReadWrite);
	    num_of_sensors++;
	    fileWrite(config_file, &num_of_sensors, 1);
	    fileClose(config_file);
	    wait_key_release = 1;

	    if (num_of_sensors == 4) {
		lcd.clear();
		lcd.setCursor(0,0);
		lcd.print("Konfiguracja");
		lcd.setCursor(0,1);
		if (num_of_sensors != 0) { lcd.print("Zapisana"); }
		    else		 { lcd.print("Skasowana"); }
//		spiffs_flush();
		spiffs_unmount();
		while (1 == 1) { } /* dummy loop, as we don't hit watchdog, reset in 5s */
				    } //num_of_sensors = 4
						    } //SOFT_KEY_1_MASK

					} // wait_key_release = 0
	#if defined(KEY_1_PIN)
	    else { if ((soft_keys & SOFT_KEY_1_MASK) == 0) { wait_key_release = 0; } }
	#else
	    else { if ((soft_keys & SOFT_KEY_2_MASK) == 0) { wait_key_release = 0; } }
	#endif
									    } // if s>3
				    } // IN_CONFIG
else { // NORMAL WORK
/* STS-21 */
	STS_Ready = I2C_bus.start((I2C_STS_ADDRESS << 1) | 0x01);

	    if (STS_Ready) {
			    STS_data[0] = I2C_bus.read(0);
			    STS_data[1] = I2C_bus.read(0);
			    STS_data[2] = I2C_bus.read(1);
			    }
		else	    {/* STS not ready */ }
	I2C_bus.stop(); // STOP

	if (sensirion_crc8(&STS_data[0], 3) == 0) {
	    I2C_bus.start(I2C_STS_ADDRESS << 1);
	    I2C_bus.write(0xF3); // No Hold Master STS Start T
	    I2C_bus.stop();		       }
	else	{
	    I2C_bus.start(I2C_STS_ADDRESS << 1);
	    I2C_bus.write(0xFE); // soft RESET
	    I2C_bus.stop();		       }
/* /STS-21 */

/* PCF 8574 */
#if defined (I2C_IO)
// DEBUG
//	if ((i2c_input2 & 0x04) == 0) { i2c_output3 |= 0x04; }
//	    else		  { i2c_output3 &= ~0x04; }
//	if ((i2c_input2 & 0x02) == 0) { i2c_output3 |= 0x02; }
//	    else		  { i2c_output3 &= ~0x02; }
//	if ((i2c_input2 & 0x01) == 0) { i2c_output3 |= 0x01; }
//	    else		  { i2c_output3 &= ~0x01; }

//	i2c_output2 = i2c_output;
//	i2c_output3 = i2c_output;

//	pwm_output = PWM_table[pwm_output_idx];
// /DEBUG

/*----------standalone converter -----------------*/

	    I2C_bus.start(I2C_IO_ADDRESS);
	    i2c_output &= I2C_IO_MASK;
	    I2C_bus.write(~i2c_output);
	    I2C_bus.stop();

	    I2C_bus.start(I2C_IO_ADDRESS | 0x01);
	    i2c_input = I2C_bus.read(1);
	    i2c_input = ~i2c_input;
	    i2c_input &= ~I2C_IO_MASK;
	    I2C_bus.stop();

/*----------- PCF on LCD board -------------------*/

	    I2C_bus.start(I2C_IO_ADDRESS2);
	    i2c_output2 &= I2C_IO_MASK2;
	    I2C_bus.write(~i2c_output2);
	    I2C_bus.stop();

	    I2C_bus.start(I2C_IO_ADDRESS2 | 0x01);
	    i2c_input2 = I2C_bus.read(1);
	    i2c_input2 = ~i2c_input2;
	    i2c_input2 &= ~I2C_IO_MASK2;
	    I2C_bus.stop();

/*----------- PCF on CPU board -------------------*/

	    I2C_bus.start(I2C_IO_ADDRESS3);
	    i2c_output3 &= I2C_IO_MASK3;
	    I2C_bus.write(~i2c_output3);
	    I2C_bus.stop();

	    I2C_bus.start(I2C_IO_ADDRESS3 | 0x01);
	    i2c_input3 = I2C_bus.read(1);
	    i2c_input3 = ~i2c_input3;
	    i2c_input3 &= ~I2C_IO_MASK3;
	    I2C_bus.stop();

/*------------------------------------------------*/

#endif
/* PCF 8574 */

	LCD_virt_display(lcd);
	
	/* 1s */
	if ((flagi & S1_FLAG) != 0) {
	    flagi &= ~S1_FLAG;

	    do_1s_stuff();
				    } /* /S1_FLAG test */

     } // IN_CONFIG = 0
			    } //500ms_ISR

#endif /* defined(__ets__) */

/* ------------------------------------------ */
